package com.simuwang.manage.api.emailTemplate;

import com.simuwang.base.common.support.MybatisPage;
import com.simuwang.base.pojo.dto.query.EmailTemplatePageQuery;
import com.simuwang.base.pojo.vo.*;
import com.simuwang.logging.SystemLog;
import com.simuwang.manage.service.EmailTemplateService;
import com.smppw.common.pojo.ResultVo;
import com.smppw.common.pojo.enums.status.ResultCode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@SystemLog(value = "邮件模版管理")
@RestController
@RequestMapping("/v1/email/template")
public class EmailTemplateController {
    @Autowired
    private EmailTemplateService emailTemplateService;

    /**
     * 邮件模版列表查询
     * @param emailTemplatePageQuery
     * @return
     */
    @SystemLog(value = "邮件模版列表查询")
    @RequestMapping("search-email-template-list")
    public MybatisPage<EmailTemplatePageVO> searchEmailTemplateList(EmailTemplatePageQuery emailTemplatePageQuery){
        return emailTemplateService.searchEmailTemplateList(emailTemplatePageQuery);
    }


    /**
     * 批量删除模版
     * @param idListVO
     * @return
     */
    @SystemLog(value = "批量删除模版",type = SystemLog.Type.DELETE)
    @RequestMapping("delete-email-template-list")
    public boolean deleteEmailTemplateList(@RequestBody IdListVO idListVO){
        emailTemplateService.deleteEmailTemplateList(idListVO);
        return true;
    }

    /**
     * 开启关闭状态
     * @param statusVO
     * @return
     */
    @SystemLog(value = "开启关闭状态", type = SystemLog.Type.UPDATE)
    @PostMapping("control-template-status")
    public ResultVo controlEmailTemplateConfig(@RequestBody StatusVO statusVO){
        ResultVo vo = new ResultVo(ResultCode.SAVE_SUCCESS.getCode());
        try{
            emailTemplateService.controlEmailTemplateConfig(statusVO);
            vo.setData(true);
        }catch (Exception e){
            vo.setCode(ResultCode.SAVE_FAILED.getCode());
            vo.setMsg(e.getMessage());
            vo.setData(false);
        }
        return vo;
    }

    /**
     * 保存模版信息
     * @param emailTemplateVO
     * @return
     */
    @SystemLog(value = "保存模版信息",type = SystemLog.Type.INSERT)
    @RequestMapping("save-email-template")
    public ResultVo saveEmailTemplate(@RequestBody EmailTemplateVO emailTemplateVO){
        ResultVo vo = new ResultVo(ResultCode.SAVE_SUCCESS.getCode());
        if(!emailTemplateService.checkEmailTemplateUnique(emailTemplateVO)){
            vo.setMsg("已存在相同的邮件模版编码，不能重复添加");
            vo.setData(false);
            return vo;
        }
        emailTemplateService.saveEmailTemplate(emailTemplateVO);
        vo.setData(true);
        return vo;
    }

    /**
     * 模版编辑查询
     * @param idVO
     * @return
     */
    @SystemLog(value = "模版编辑查询")
    @GetMapping("search-email-template-info")
    public ResultVo searchEmailTemplate(IdVO idVO){
        ResultVo vo = new ResultVo(ResultCode.SAVE_SUCCESS.getCode());
        try{
            EmailTemplateVO emailTemplateVO = emailTemplateService.searchEmailTemplate(idVO.getId());
            vo.setData(emailTemplateVO);
        }catch (Exception e){
            vo.setCode(ResultCode.SAVE_FAILED.getCode());
            vo.setData(false);
        }
        return vo;
    }

}
