package com.simuwang.manage.service.impl;

import com.simuwang.base.common.support.MybatisPage;
import com.simuwang.base.common.util.StringUtil;
import com.simuwang.base.mapper.daq.*;
import com.simuwang.base.pojo.dos.FundAliasDO;
import com.simuwang.base.pojo.dos.FundNotMappingInfoDO;
import com.simuwang.base.pojo.dos.FundRegisterDO;
import com.simuwang.base.pojo.dto.query.FundAliasPageQuery;
import com.simuwang.base.pojo.vo.FundAliasVO;
import com.simuwang.base.pojo.vo.IdListVO;
import com.simuwang.base.pojo.vo.ReviewStatusVO;
import com.simuwang.manage.service.EmailFundAssetService;
import com.simuwang.manage.service.EmailFundNavService;
import com.simuwang.manage.service.EmailFundInfoService;
import com.simuwang.manage.service.FundAliasService;
import com.simuwang.shiro.utils.UserUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * FileName: FunAliasServiceImpl
 * Author:   chenjianhua
 * Date:     2024/9/14 16:18
 * Description: ${DESCRIPTION}
 */
@Service
public class FundAliasServiceImpl implements FundAliasService {
    @Autowired
    private FundAliasMapper fundAliasMapper;

    @Autowired
    private EmailFundNavMapper emailFundNavMapper;

    @Autowired
    private EmailFundAssetMapper emailFundAssetMapper;

    @Autowired
    private CompanyInformationMapper companyInformationMapper;

    @Autowired
    private EmailFundInfoService emailFundInfoService;

    @Autowired
    private EmailFundNavService emailFileNavService;

    @Autowired
    private EmailFundAssetService emailFundAssetService;

    @Autowired
    private FundNotMappingMapper fundNotMappingMapper;

    @Override
    public MybatisPage<FundAliasVO> searchFundAlias(FundAliasPageQuery fundAliasPageQuery) {
        List<FundAliasDO> fundAliasDOList = fundAliasMapper.searchFundAlias(fundAliasPageQuery);
        List<FundAliasVO> fundAliasVOList =  fundAliasDOList.stream().map(FundAliasDO::toVo).collect(Collectors.toList());
        long total = fundAliasMapper.countFundAlias(fundAliasPageQuery);
        return MybatisPage.of(total,fundAliasVOList);
    }

    @Override
    public FundAliasVO searchFundAliasById(Integer id) {
        FundAliasDO fundAliasDO = fundAliasMapper.searchFundAliasById(id);
        if(StringUtil.isNull(fundAliasDO)){
            return null;
        }
        return fundAliasDO.toVo();
    }

    @Override
    @Async
    public void saveFundAlias(FundAliasVO fundAliasVO) {
        FundAliasDO fundAliasDO = new FundAliasDO();
        fundAliasDO.setSourceFundName(fundAliasVO.getSourceFundName());
        fundAliasDO.setTargetFundId(fundAliasVO.getTargetFundId());
        fundAliasDO.setSourceRegisterNumber(fundAliasVO.getSourceRegisterNumber());
        fundAliasDO.setTargetFundName(fundAliasVO.getTargetFundName());
        fundAliasDO.setTargetRegisterNumber(fundAliasVO.getTargetRegisterNumber());
        fundAliasDO.setId(fundAliasVO.getId());
        fundAliasDO.setIsvalid(1);
        fundAliasDO.setUpdateTime(new Date());
        if(StringUtil.isNotEmpty(fundAliasVO.getTargetFundId())){
            fundAliasDO.setReviewStatus(1);
        }
        FundAliasDO oldFundAliasDO = fundAliasMapper.searchFundAliasById(fundAliasVO.getId());
        String oldFundId = oldFundAliasDO.getTargetFundId();
        fundAliasMapper.saveFundAlias(fundAliasDO);
        //将采集的净值数据匹配之后重新入库
        emailFileNavService.reparseFileNav(fundAliasVO.getSourceFundName(),fundAliasVO.getSourceRegisterNumber(),fundAliasVO.getTargetFundId(),oldFundId);
        //将采集的规模数据匹配之后重新入库
        emailFundAssetService.reparseFileAsset(fundAliasVO.getSourceFundName(),fundAliasVO.getSourceRegisterNumber(),fundAliasVO.getTargetFundId(),oldFundId);
        //处理采集数据，将采集的数据入库
        emailFundInfoService.reparseValuationFile(fundAliasVO.getSourceFundName(),fundAliasVO.getSourceRegisterNumber());
        //更新别名统计数据
        computeFundMappingInfo(fundAliasVO.getSourceFundName(),fundAliasVO.getSourceRegisterNumber());
    }

    private void computeFundMappingInfo(String fundName,String registerNumber){
        FundNotMappingInfoDO fundNotMappingInfoDO = new FundNotMappingInfoDO();
        fundNotMappingInfoDO.setFundName(fundName);
        fundNotMappingInfoDO.setRegisterNumber(registerNumber);
        //异常净值数量
        Integer navTotal = emailFundNavMapper.countNoStoreNav(fundName,registerNumber);
        //异常净值数量
        Integer assetTotal = emailFundAssetMapper.countNoStoreAsset(fundName,registerNumber);
        String lastPriceDate = emailFundNavMapper.getLastPriceDateByfundRegisterNumber(fundName,registerNumber);
        fundNotMappingInfoDO.setAssetTotal(assetTotal);
        fundNotMappingInfoDO.setLastPriceDate(lastPriceDate);
        fundNotMappingInfoDO.setNavTotal(navTotal);
        fundNotMappingInfoDO.setIsvalid(1);
        fundNotMappingInfoDO.setCreateTime(new Date());
        fundNotMappingInfoDO.setUpdateTime(new Date());
        FundNotMappingInfoDO oldFundNotMappingInfoDO = fundNotMappingMapper.getFundNotMappingInfo(fundNotMappingInfoDO);
        if(oldFundNotMappingInfoDO != null){
            fundNotMappingInfoDO.setId(oldFundNotMappingInfoDO.getId());
        }
        fundNotMappingMapper.insertOrUpdate(fundNotMappingInfoDO);
    }


    @Override
    public void deleteFundAlias(IdListVO idListVO) {
        List<Integer> idList = idListVO.getIdList();
        Integer userId = UserUtils.getLoginUser().getUserId();
        if(idList.size() > 0){
            fundAliasMapper.batchDelete(idList,userId);
        }
    }

    @Override
    public List<FundRegisterDO> getNotMappingFundList() {
        return fundAliasMapper.getNotMappingFundList();
    }

    @Override
    public void updateReviewStatus(ReviewStatusVO reviewStatusVO) {
        Integer userId = UserUtils.getLoginUser().getUserId();
        fundAliasMapper.updateReviewStatus(reviewStatusVO.getIdList(),reviewStatusVO.getReviewStatus(),userId);
    }
}
